// s2m.c

// Takes any number of parameters greater than three, as long as they are integers;
// constructs a difference table, determines order of the sequence (if polynomial)
// and spits back an augmented matrix allowing calculation of coefficients.

// For instance, consider the sequence:
// 1,2,3,4
// s2m(1,2,3,4) = (1,1,1)
//                (2,1,2)
//                (3,1,3)
//                (4,1,4)
// This shows that the sequnce is of order 1 (ie, it can be expressed T(n) = a*n^1 + b*n^0).
// Thus, the matrix corresponds to a system of linear equations that can be solved for
// the values of a and b using gaussian elimination. The rref([matrix]) function is
// a recommended next step after use of this function.

// Alex Wilson, 2006.

#define USE_TI89
#define USE_TI92PLUS
#define USE_V200

#define MIN_AMS 200
#define RETURN_VALUE
#define SAVE_SCREEN

#include <tigcclib.h>

void _main(void)
{
	ESI arg_ptr = top_estack;

	int list_length=0, i, ii;
	int *list;
	int *thisrow, thisrow_l, order=0;
	short cont=1;

	// count our arguments
	while (GetArgType(arg_ptr) != END_TAG) {
		if (GetArgType(arg_ptr) == POSINT_TAG || GetArgType(arg_ptr) == NEGINT_TAG)
			list_length++;

		arg_ptr = next_expression_index(arg_ptr);
	}

	// check for < 3
	// don't even bother trying when we only have two
	// TODO: return a proper error code rather than string
	if (list_length < 3) {
		push_zstr("ERROR: needs >= 3 args");
		return;
	}

	// allocate memory for the array
	list = malloc((list_length+1) * sizeof(int));

	// get arguments as an array of singed ints
	InitArgPtr(arg_ptr);

	for (i=0; i<list_length; i++) {
		if (GetArgType(arg_ptr) == NEGINT_TAG) {
			list[i] = -1 * GetIntArg(arg_ptr);
		} else {
			list[i] = GetIntArg(arg_ptr);
		}
	}

	// clear the exp. stack
	while (ESTACK(top_estack) != END_TAG)
    		top_estack = next_expression_index (top_estack);
  	top_estack--;

	// find order of sequence (using table of differences)

	// first, make a temp copy to work with
	// (we'll process it in-place for speed and memory)
	thisrow = malloc(sizeof(int)*list_length);
	memcpy(thisrow, list, sizeof(int)*list_length);

	thisrow_l = list_length -1;

	while (1)
	{
		// check if we have a zero row yet
		cont = 0;
		for (i=0; i<=thisrow_l; i++)
			if (thisrow[i] != 0) cont = 1;
		// if we have a zero row, get out of the loop
		if (cont == 0) break;

		// or else, make the next row in-place
		order++;

		thisrow_l--;
		for (i=0; i<=thisrow_l; i++) {
			thisrow[i] = abs(thisrow[i] - thisrow[i+1]);
		}
	}
	
	free(thisrow);

	// a matrix can be returned as just a list of lists
	
	// start the big list with an end tag
	push_END_TAG();
	// remember that we have to put it out backwards
	for (i=list_length-1; i>=0; i--) {
		push_END_TAG();
		// each row ends with the augmented column
		push_longint(list[i]);
		for (ii=0; ii<order; ii++) {
			push_longint(pow(i+1,ii));
		}
		push_LIST_TAG();
	}
	push_LIST_TAG();

	free(list);
}

